/*
* Copyright 2019 Nu-book Inc.
* Copyright 2023 Axel Waggershauser
*/
// SPDX-License-Identifier: Apache-2.0

#pragma once

#define ZXING_READERS
#define ZXING_WRITERS

// Version numbering
#define ZXING_VERSION_MAJOR 2
#define ZXING_VERSION_MINOR 3
#define ZXING_VERSION_PATCH 0

#define ZXING_VERSION_STR "2.3.0"
